//
//
//		Name:		cEnemy.h
//		Date:		19-02-2009
//		Author:		Alejandro Santiago
//		Email:		alejandro@aeonphyxius.com
//		Desc:		Header of the class cEnemy. This class encapsulate all logic
//					about the enemies that the player can find and fight
//
//

#ifndef __ENEMY_H__
#define __ENEMY_H__

#include "Globals.h"
#include "cMap.h"

#define ENEMY_SIZE 50
#define ENEMY_DELAY 0
#define ENEMY_ENABLE 0
#define ENEMY_DISABLE 1

class cEnemy
{

private:
	int add_value;					//
	auto_ptr<cTexture> pTexture;	// Enemy's texture
	int posx;						// Enemy's x position	
	int posy;						// Enemy's y position
	int direction;					// Enemy's direction
	float inix;						// Enemy's initial x position in texture file
	float endx;						// Enemy's end x position in texture file
	float iniy;						// Enemy's initial y position in texture file
	float endy;						// Enemy's end y position in texture file
	int enemy_delay;				// Enemy's delay in animation
	int initialposx;				// Enemy's inital position
	int prev_map_pos;				// Enemy's inital position before update
	int enable;						// is it enabled?
public:
	
	// Constructor and destructor
	cEnemy();
	cEnemy(int _addValue,int _enemyType,int _posx, int _posy);
	~cEnemy();

	/* 
	* Enemy's logic to update, check collisions, etc
	*
	* @param _map level's map
	*/
	void logic(const cMap * _map);

	/* 
	* Check if the enemy collides against the floor
	*
	* @param _map level's map
	* @return  true if the enemy collides against the floor
	*/
	bool collidesMapFloor(const cMap * _map);

	/* 
	* Move Enemy to the right
	*
	* @param _map level's map
	*/
	void moveRigth(const cMap * _map);
	
	/* 
	* Move Enemy to the left
	*
	* @param _map level's map
	*/
	void moveLeft(const cMap * _map);
	
	/* 
	* Check if the enemy collides against the wall
	*
	* @param _map level's map
	* @return  true if the enemy collides against the wall
	*/
	bool collidesMapWall(const cMap * _map);

	
	// Getters
	//void setAdd_value(int add_value);
	int getAddValue()const;
	cTexture* getTextureEnemy() const;
	float getEndx() const;
	float getEndy() const;
	float getInix() const;
	float getIniy() const;
	int getPosx() const;
	int getPosy() const;
	int getEnemyTileSize() const;
	int getEnable() const;
	int getRelativePosx(const int _mapx) const;
	int getRelativePosy() const;

};

#endif; // __ENEMY_H__