#include "cGame.h"

// Singleton instance
cGame* cGame::instance = NULL;

// Singleton implementation
cGame *cGame::getInstance()
{	
	if (instance == NULL)
	{
		instance = new cGame();
	}
    return instance;
}


cGame::cGame()
{	
	for (int i=0;i< 256; i++)
	{
		keys[i] =0;
	}
}

cGame::~cGame()
{
}


bool cGame::Initialize()
{
	bool res=true;
	game_status = GAME_PLAY ;
	menu_pos = 0;	

	//Graphics initialization
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,GAME_WIDTH,0,GAME_HEIGHT,0,1);
	glMatrixMode(GL_MODELVIEW);
	//Alpha test enabled
	glAlphaFunc(GL_GREATER, 0.05f);
	glEnable(GL_ALPHA_TEST);
	//Textures enabled
	glEnable(GL_TEXTURE_2D);
	
	//Scene initialization
	res = cGameCore::getInstance()->getScene()->LoadLevel(1);	
	
	return res;

	
	
}

bool cGame::Loop()
{
	bool res=true;
	
	switch (game_status)
	{
	case GAME_MENU:
		res = ProcessMenu();
		if(res)
		{
			Render();
		}
		break;
	case GAME_PLAY:
		res = Process();
		if(res) 
		{
			Render();
		}
		break;
	case GAME_EXIT:
		res=false;
		break;		
	}	
	return res;
}

void cGame::Finalize()
{
}

//Input
void cGame::ReadKeyboard(unsigned char key, int x, int y, bool press)
{
	keys[key] = press;	
}

void cGame::ReadMouse(int button, int state, int x, int y)
{
}


bool cGame::ProcessMenu()
{
	bool res=true;
	//Process Input
	if(keys[27])	
	{
		res=false;	
	}
	
	if(keys[GLUT_KEY_UP]) // Key UP => move up position in th menu
	{
		if(menu_pos !=0)
		{
			menu_pos--;	
		}
	}

	if(keys[GLUT_KEY_DOWN]) // Key Down => move down position in th menu
	{
		if(menu_pos !=3)
		{
			menu_pos++;
		}
	}

	if(keys[13]) // Key Enter => select menu option
	{
		switch (menu_pos)
		{
		case MENU_OPTION_1PLAYER:
			game_status=GAME_PLAY;
			break;
		case MENU_OPTION_EXIT:
			game_status=GAME_EXIT;
			break;
		case MENU_OPTION_OPIONS:
			game_status=GAME_OPTIONS;
			break;
		case MENU_OPTION_HELP:
			game_status=GAME_HELP;
			break;
		}
	}
	return res;
}


bool cGame::Process()
{
	using namespace std;
	bool res=true;
	
	vector<cEnemy*> enemies  = cGameCore::getInstance()->getScene()->getEnemies(); // Get enemies from the Scene to work with them
	vector<cItem*> items  = cGameCore::getInstance()->getScene()->getItems(); // Get items from the Scene to work with them
	cMap* map = cGameCore::getInstance()->getScene()->getMap(); // Get the map to work with locally

	//Process Input
	if(keys[27])	
		res=false;
	
	if(keys[GLUT_KEY_UP] && (cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_WALK || cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_ATTACK )) 
	{
		cGameCore::getInstance()->getPlayer()->jump();	
	}
	
	if(keys[32] || cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_ATTACK)// SPACE (ATTACK)	
	{
		cGameCore::getInstance()->getPlayer()->attack();
	}
	
	if(keys[GLUT_KEY_RIGHT] && cGameCore::getInstance()->getPlayer()->getPlayerStatus()!=PLAYER_STATUS_ATTACK)// RIGTH (MOVE RIGTH)
	{	cGameCore::getInstance()->getPlayer()->setDirection(DIR_RIGHT);
		cGameCore::getInstance()->getPlayer()->moveRigth();
		for(int i=0;i<10;i++)
		{
			items[i]->moveRigth(map);
		}
		for(int i=0;i<5;i++)
		{
			enemies[i]->moveRigth(map);
		}
	}
	if(keys[GLUT_KEY_LEFT] && cGameCore::getInstance()->getPlayer()->getPlayerStatus()!=PLAYER_STATUS_ATTACK)//LEFT
	{	
		cGameCore::getInstance()->getPlayer()->setDirection(DIR_LEFT);
		cGameCore::getInstance()->getPlayer()->moveLeft();		
		
		for(int i=0;i<items.size();i++)
		{
			items[i]->moveLeft(map);
		}
		for(int i=0;i<enemies.size();i++)
		{
			enemies[i]->moveLeft(map);
		}
	}		

	// items logic
	for(int i=0;i<items.size();i++)
	{
		items[i]->logic(map);
	}

	// enemies logic
	for(int i=0;i<enemies.size();i++)
	{
		enemies[i]->logic(map);
	}

	// Player logic
	cGameCore::getInstance()->getPlayer()->logic();
	
	return res;
}


//Output
void cGame::Render()
{
	glClear(GL_COLOR_BUFFER_BIT);	
	glLoadIdentity();
	switch (game_status)
	{
	case GAME_MENU:
		cGameCore::getInstance()->getScene()->DrawMenu(menu_pos);
		break;
	case GAME_PLAY:
		cGameCore::getInstance()->getScene()->Draw();
		break;
	}
	glutSwapBuffers();
}