//
//
//		Name:		cGame.h
//		Date:		01-02-2009
//		Author:		Alejandro Santiago
//		Email:		alejandro@aeonphyxius.com
//		Desc:		Game class, application core. Implementing the singleton
//					pattern, will contain all games variables with the game
//					logic
//
//

#ifndef __GAME_H__
#define __GAME_H__

//#include <iostream>// to be removed (only to use printf function)
#include "cPlayer.h"
#include "cMap.h"
#include "cScene.h"
#include "cGameCore.h"
#include "cEnemy.h"
#include "Globals.h"



class cGame
{

private: // Class variables
	
	unsigned char keys[256];		// keys arrays mapping the current keyboard status	
	int game_status;				// game status
	int menu_pos;					// when in menu, which position are we?


public:
	/**
    *
	*
	* @return  Unique instance of this class
    */
	bool Initialize();
	
	/**
	* Main game loop. 
	*
	* @return 
    */
	bool Loop();
	
	/**
	* After game's loop, clean up.
	*
    */
	void Finalize();
	
	/**
    * Update function for menu state
	*
	* @return true if the update process went correct, otherwise false
    */
	bool ProcessMenu(void);
	
	/**
    * Keyboard handler. // TBD: Move this into an Input class
	* 
	* @param _key key that trigered the action
	* @param _press is it pressed?
	* @param _x key x-coordinate position
	* @param _y key y-coordinate position
    */	
	void ReadKeyboard(unsigned char _key, int _x, int _y, bool _press);
	
	/**
    * Mouse handler. // TBD: Move this into an Input class
	* 
	* @param _button mouse button that trigered the action
	* @param _state mouse buttons state
	* @param _x mouse x-coordinate position
	* @param _y mouse y-coordinate position
    */
	void ReadMouse(int _button, int _state, int _x, int _y);
	

	/**
    * Update function called every update
	*
	* @return true if the update process went correct, otherwise false
    */
	bool Process();
		
	/**
    * This method renders all objects depending on last updates
	*
    */
	void Render();
	


public:// Singleton implementation
	/**
    * Will return the unique instance of this class, if does not
	* exists creates a new object (and unique). 
	*
	* @return  Unique instance of this class
    */
	static cGame *getInstance();

	/**
    * Destructor
    */
	~cGame();
private:
    /**
    * Private constructor, controled by our singleton implementation.
	* This class wont be instantiated.
    */
	cGame();

	/* unique instance of this class */
	static cGame *instance;

};

#endif  // __GAME_H__