//
//
// Name:		cGame.h
// Date:		01-02-2009
// Author:		Alejandro Santiago
// Email:		alejandro@aeonphyxius.com
// Desc:		Game class, application core. Implementing the singleton
//				pattern, will contain all games variables with the game
//				logic
//
//

#ifndef __GAME_CORE_H__
#define __GAME_CORE_H__

#include "cPlayer.h"
#include "cScene.h"
#include "globals.h"
#include "cFileLoader.h"
//#include "cEnemy.h"



class cGameCore
{

private : // Class variables
	
	auto_ptr<cScene> pScene;			// Game scene
	auto_ptr<cPlayer> pPlayer;			// Player data
	auto_ptr<cFileLoader> pFileLoader;	// File I/O operations

public:

	/**
    * Return the current player's information
	*
	* @return player information
    */
	cPlayer * getPlayer() const;
	
	/**
	* Returns the current game scene with all game data
	*
	* @return game scene
    */
	cScene  * getScene() const;

	void InitializeGameComponents(const int _playerType);


public:// Singleton implementation
	/**
    * Will return the unique instance of this class, if does not
	* exists creates a new object (and unique). 
	*
	* @return  Unique instance of this class
    */
	static cGameCore *getInstance();

	/**
    * Destructor
    */
	~cGameCore();
private:
    /**
    * Private constructor, controled by our singleton implementation.
	* This class wont be instantiated.
    */
	cGameCore();

	/* unique instance of this class */
	static cGameCore *instance;

};

#endif  // __GAME_CORE_H__
