//
//
//		Name:		cItem.h
//		Date:		14-02-2009
//		Author:		Alejandro Santiago
//		Email:		alejandro@aeonphyxius.com
//		Desc:		All items in our scene. Will have a texture. Also manages its ouwn colliding methods
//
//
#ifndef __ITEM_H__
#define __ITEM_H__

#include "Globals.h"
#include "cTexture.h"
#include "cMap.h"

#define ITEM_SIZE 20
#define ITEM_DELAY 0

class cItem
{
private: // Class variables
	int add_value;							//
	auto_ptr <cTexture> pTexture;			// Item's texture image
	int posx;								// X position
	int posy;								// Y position
	int item_type;							// item type (extra life, points, etc)
	float inix;								// initial x position on texture
	float endx;								// end x position on texture
	float iniy;								// initial y position on texture
	float endy;								// end y position on texture
	int item_delay;							// delay on this item animation
	int initialposx;						// initial x position
	int prev_map_pos;						// previous map position
	int enable;								// is it enabled?

public:
	
	// Constructors and destructor
	cItem();
	cItem(int _addValue,int _itemType,int _posx, int _posy);
	~cItem();

	/* 
	* Method to check the item pos and the map
	*
	* @param: _map from our scene, where we locate our items
	*/
	void logic(cMap * _map);

	/* 
	* Check if this item collides against the given map
	*
	* @param map to check if this item collides
	* @return true if collides or false if not
	*/
	bool collidesMapFloor(cMap * _map);

	/* 
	* when moving right, update the item position
	*
	* param: _map Scene's map
	*/
	void moveRigth(cMap * _map);

	/* 
	* when moving left, update the item position
	*
	* param: _map Scene's map
	*/
	void moveLeft(cMap * _map);

	// setters
	void setAddAalue(int _addValue);	
	//void setTexture_item(cTexture texture_item);
	void setEnable(int _enable);
	
	// getters
	int getAddValue() const;
	cTexture* getTextureItem() const;
	float getEndx() const;
	float getEndy() const;
	float getInix() const;
	float getIniy() const;
	int getPosx() const;
	int getPosy() const;
	int getItemTileSize() const;
	int getEnable() const;
	int getRelativePosx() const;
	int getRelativePosy() const;
};

#endif; // __ITEM_H__