//
//
//		Name:		cPlayer.cpp
//		Date:		12-02-2009
//		Author:		Alejandro Santiago
//		Email:		alejandro.santiago@gmail.com / asantiago@uoc.edu
//		Desc:		Class cPlayer that contains all player's information, and update / render methods
//                  and update / render methods
//
//
#include "cPlayer.h"
#include "cGameCore.h"

cPlayer::cPlayer()
{
	pTtexture = auto_ptr<cTexture> (new cTexture());
	frame_seq = 0;
	frame_delay = 0;
	direction=DIR_RIGHT;	
	sTtextureFile = "img/tryius_heroe.png";
	posx=PLAYER_INI_POS_X;
	posy=PLAYER_INI_POS_Y;
	relative_posx=PLAYER_INI_POS_X;
	relative_posy=PLAYER_INI_POS_Y;
	despx = 0.061;
	despy=0.115;
	player_status = PLAYER_STATUS_WALK;
}

cPlayer::cPlayer(const int _playerType)
{
	pTtexture = auto_ptr<cTexture> (new cTexture());
	num_lives=5;
	player_type=player_type;
	frame_seq = 0;
	frame_delay = 0;
	direction=DIR_RIGHT;	
	posx=PLAYER_INI_POS_X;
	posy=PLAYER_INI_POS_Y;
	player_status = PLAYER_STATUS_WALK;
	relative_posx=PLAYER_INI_POS_X;
	relative_posy=PLAYER_INI_POS_Y;

	switch(_playerType)
	{
	case PLAYER_TYPE_GUILLIUS:
		sTtextureFile="img/guillius_heroe.png";
		despx=GUILLIUS_DESP_X;
		despy=GUILLIUS_DESP_Y;	
		break;
	case PLAYER_TYPE_AX:
		sTtextureFile="img/ax_heroe.png";
		despx=AX_DESP_X;
		despy=AX_DESP_Y;		
		break;
	case PLAYER_TYPE_TRYIUX:
		sTtextureFile="img/tryiux_heroe.png";
		despx=TRYIUX_DESP_X;
		despy=TRYIUX_DESP_Y;
		break;
	}
}

cPlayer::~cPlayer()
{
}


void cPlayer::jump()
{
	if(player_status!=PLAYER_STATUS_JUMPING)
	{
		if(collidesMapFloor(cGameCore::getInstance()->getScene()->getMap()))
		{
			player_status=PLAYER_STATUS_JUMPING;
			jump_alfa = 0;
			jump_y = posy;
		}
	}
}


void cPlayer::logic()
{
	float alfa;
	cMap *map = cGameCore::getInstance()->getScene()->getMap();

	// Player position
	if((player_status!=PLAYER_STATUS_JUMPING) && (player_status!=PLAYER_STATUS_DOWN))
	{
		if(!collidesMapFloor(map))
		{
			posy--;
			relative_posy--;
		}
	}
	else
	{
		if(player_status==PLAYER_STATUS_JUMPING)
		{
			jump_alfa += JUMP_STEP;
			
			if(jump_alfa == 180)
			{
				player_status = PLAYER_STATUS_DOWN;				
				posy = jump_y;
				relative_posy = jump_y;

			}
			else
			{
				alfa = ((float)jump_alfa) * 0.017453f;

				relative_posy = jump_y + (int)( ((float)JUMP_HEIGHT) * sin(alfa) );
				posy = jump_y + (int)( ((float)JUMP_HEIGHT) * sin(alfa) );
			
				if(jump_alfa > 90)
				{
					//Over floor?
					if(collidesMapFloor(map))
						player_status = PLAYER_STATUS_WALK;			
				}
			}
		}
		else if(player_status==PLAYER_STATUS_DOWN)
		{
			//Over floor?
			if(!collidesMapFloor(map))
			{
				relative_posy -= (2*STEP_LENGTH);
				posy -= (2*STEP_LENGTH);
			}
			else player_status = PLAYER_STATUS_WALK;				
		}
	}
/*
	// Player -- Item interacions

	for (int i=0;i<10;i++)
	{
		if(items[i]->getEnable()==ITEM_ENABLE)
		{
			if(collidesItem(items[i]))
			{
				items[i]->setEnable(ITEM_DISABLE);
				num_lives++;
			}			

		}
	}
	// Player -- Enemies interacions
	for (int i=0;i<10;i++)
	{
		if(enemies[i]->getEnable()==ENEMY_ENABLE)
		{
			if(collidesEnemy(map,enemies[i]))
			{
				enemies[i]->setEnable(ENEMY_DISABLE);
				if(!player_status== PLAYER_STATUS_ATTACK)// If the player it is not in attack mode...
					num_lives--;
			}			

		}
	}*/
}

bool cPlayer::collidesEnemy(cMap * _map,cEnemy * _enemy)
{
	int pos_tile_x,pos_tile_y;
	int pos_item_tile_x,pos_item_tile_y;

	// Take the relative tile size position of the player
	pos_tile_x = relative_posx / TILE_SIZE; 
	pos_tile_y = relative_posy / TILE_SIZE;
	// Take the relative tile size position of the item
	pos_item_tile_x = _enemy->getRelativePosx(_map->getCoordX()) / TILE_SIZE; 
	pos_item_tile_y = _enemy->getRelativePosy() / TILE_SIZE;

	// If right we want to check the next one, if left the previous one
	/*if(direction==DIR_RIGHT) 
		pos_tile_x += 1;*/
	
	// Check
	if ((pos_tile_x==pos_item_tile_x )&&(pos_tile_y==pos_item_tile_y))
	{
		return true;	
	}
	else
	{
		return false;	
	}
}


bool cPlayer::collidesItem(cItem * _item)
{
	int pos_tile_x,pos_tile_y;
	int pos_item_tile_x,pos_item_tile_y;

	// Take the relative tile size position of the player
	pos_tile_x = relative_posx / TILE_SIZE; 
	pos_tile_y = relative_posy / TILE_SIZE;
	// Take the relative tile size position of the item
	pos_item_tile_x = _item->getRelativePosx() / TILE_SIZE; 
	pos_item_tile_y = _item->getRelativePosy() / TILE_SIZE;

	// If right we want to check the next one, if left the previous one
	/*if(direction==DIR_RIGHT) 
		pos_tile_x += 1;*/
	
	// Check
	if ((pos_tile_x==pos_item_tile_x )&&(pos_tile_y==pos_item_tile_y))
	{
		return true;	
	}
	else
	{
		return false;
	}
}


bool cPlayer::collidesMapWall(const cMap * _map)
{
	int pos_tile_x,pos_tile_y;

	// Take the relative tile size position 
	pos_tile_x = relative_posx / TILE_SIZE; 
	pos_tile_y = relative_posy / TILE_SIZE;
	

	// If right we want to check the next one, if left the previous one
	if(direction==DIR_RIGHT) 
		pos_tile_x += 1;
	
	// Check
	if (_map->getMapPos( pos_tile_x,pos_tile_y )!= 0 )
	{
		return true;	
	}
	else 
	{
		return false;	
	}
}


bool cPlayer::collidesMapFloor(const cMap * _map)
{
	int pos_tile_x,pos_tile_y;

	// Take the relative tile size position 
	pos_tile_x = relative_posx / TILE_SIZE; 
	pos_tile_y = (relative_posy-1) / TILE_SIZE;// -1 to check the next tile (under the heroe foods)
		
	// Check
	if (_map->getMapPos(pos_tile_x,pos_tile_y)!= 0 )
	{
		return true;
	}
	else
	{
		return false;
	}
}


void cPlayer::attack()
{
		if(player_status !=PLAYER_STATUS_ATTACK)
		{
			player_status=PLAYER_STATUS_ATTACK;// attack mode
			frame_seq=0;// Restart the sequence
		}
		else
		{
			frame_delay++;
			if(frame_delay >= PLAYER_DELAY)
			{
				frame_delay=0;
				frame_seq++;				
				if(frame_seq == 4)
				{
					frame_seq=0;
					player_status=PLAYER_STATUS_WALK;
				}		
			}
		}
}


void cPlayer::moveLeft()
{	
	cMap *map  =cGameCore::getInstance()->getScene()->getMap();

	frame_delay++; 
	//Player delay
	if(frame_delay >= PLAYER_DELAY)
	{
		frame_delay=0;		
		frame_seq++;
		if(frame_seq == 4)
			frame_seq=0;//frame_seq = 0;
			
	}		
	if(!collidesMapWall(map)) // If doens't exist a map collides, update the player pos
	{
		
		//Player position
		if(posx > 160 && map->getCoordX()<0)
		{
			posx-=2;
			relative_posx-=2;
		}
		else
		{
			if(map->getCoordX()>0)
			{
				if(posx>5)
				{
					posx-=2;				
					relative_posx-=2;
				}
			}
			else
			{		
				map->setCoordX(map->getCoordX()+2);
				relative_posx-=2;
			}
		}
	}
}


void cPlayer::moveRigth()
{
	cMap *map  =cGameCore::getInstance()->getScene()->getMap();
	
	frame_delay++;

	//Player delay
	if(frame_delay >= PLAYER_DELAY)
	{
		frame_delay=0;
		frame_seq++;		
		if(frame_seq == 4)
			frame_seq=0;			
	}
	if(!collidesMapWall(map)) // If doens't exist a map collides, update the player pos
	{			
		//Player position	
		if(	posx < 160 && map->getCoordX()>-2024)
		{
			posx+=2;
			relative_posx+=2;
		}
		else
		{
			if(map->getCoordX()<-2024)
			{
				if(posx< 2004)
				{
					posx=posx+2;		
					relative_posx+=2;
				}
			}
			else
			{		
				map->setCoordX(map->getCoordX()-2);
				relative_posx+=2;
			}
		}
	}
}


void cPlayer::loadTexture()
{
	pTtexture->Load(sTtextureFile.c_str(),GL_RGB);
}




// Getters
int cPlayer::getPlayerStatus() const
{
	return player_status;
}

void cPlayer::setDirection(int direction)
{
	direction=direction;
}
float cPlayer::getDespx() const
{
	if (player_status!=PLAYER_STATUS_ATTACK)
		return despx;
	else
		return despx*1.9;
}

float cPlayer::getTileSizeX() const
{
	if (player_status!=PLAYER_STATUS_ATTACK)
		return TILE_SIZE_WIDTH;
	else
		return TILE_SIZE_ATT_WIDTH;
}

float cPlayer::getTileSizeY() const
{	
		return TILE_SIZE_HEIGHT;
}


float cPlayer::getDespy() const
{
	if (player_status!=PLAYER_STATUS_ATTACK)
		return AX_DESP_Y;
	else
		return 0.14;
}
int cPlayer::getNumLives() const
{
	return num_lives;
}

int cPlayer::getPosx() const
{
	return posx;
}
int cPlayer::getPosy() const
{
	return posy;
}
cTexture * cPlayer::getTexturePlayer() const
{
	return pTtexture.get();
}
int cPlayer::getDirection() const
{
	return direction;
}
int cPlayer::getFrameSeq() const
{
	return frame_seq; 
}