//
//
//		Name:		cPlayer.h
//		Date:		12-02-2009
//		Author:		Alejandro Santiago
//		Email:		alejandro.santiago@gmail.com / asantiago@uoc.edu
//		Desc:		Class to encapsulate the Player information and rendering/update actions.
//
//

#ifndef __PLAYER_H__
#define __PLAYER_H__

#include "cTexture.h"
#include "Globals.h"
#include "cMap.h"
#include "cItem.h"
#include "cEnemy.h"

#define PLAYER_DELAY    20
#define JUMP_STEP		4
#define STEP_LENGTH		2
#define JUMP_HEIGHT		96

#define PLAYER_INI_POS_X 160
#define PLAYER_INI_POS_Y 120

#define TILE_SIZE_WIDTH	 30
#define TILE_SIZE_ATT_WIDTH	 60
#define TILE_SIZE_HEIGHT 50

// Image displacement for the diferents heroes
#define AX_DESP_X 0.0625
#define AX_DESP_Y 0.123
#define AX_DESP_ATT_Y 0.13

#define TRYIUX_DESP_X 0.0625
#define TRYIUX_DESP_Y 0.125

#define GUILLIUS_DESP_X 0.0625
#define GUILLIUS_DESP_Y 0.125

// Class definition
class cPlayer
{
private: // Class variables

	auto_ptr <cTexture> pTtexture;	// Player's texture file
	string sTtextureFile;			// Player's texture file name
	float despx;					// displacement in x-Axis when walking
	float despy;					// displacement in x-Axis when walkin
	float despx_attack;				// displacement in x-Axis when attacking
	float despy_attack;				// displacement in x-Axis when attacking
	int player_type;				// 
	int num_lives;
	int frame_delay;
	int frame_seq;
	int direction;	
	int posx;
	int relative_posx;
	int posy;
	int relative_posy;
	int player_status;
	int jump_y;
	int jump_alfa;

public:
	//Constructor and destructor
	cPlayer();	
	cPlayer(const int player_type);		
	~cPlayer();
	
	/* 
	* Method to check if player and map collides(Wall)
	*
	* @param _map  Scene's map
	* @return true if collides or false if not
	*/
	bool collidesMapWall(const cMap * _map);
	
	/* 
	* Method to check if player and map collides(Floor)
	*
	* @param _map  Scene's map
	* @return true if collides or false if not
	*/
	bool collidesMapFloor(const cMap * _map);

	/* 
	* Scrolling right when the player is moving on this direction
	*
	*/
	void moveRigth();

	/* 
	* Scrolling left when the player is moving on this direction
	*
	*/
	void moveLeft();
	
	/* 
	* attack action
	*
	*/
	void attack();

	/* 
	* Scenen logic. Update position
	*
	*/
	void logic();

	/* 
	* Jump action
	*
	*/
	void jump();

	/* 
	* Method to check if the player collides with the items
	*
	* @param _item checks if the player collides against the given item
	* return: true if collides or false if not
	*/
	bool collidesItem(cItem * _item);

	/* 
	* checks if the given enemy collides against the given map
	*
	* @param _enemy to check if collides against the map
	* @param _map to check if the enemy collides against 
	* return: true if collides or false if not
	*/
	bool collidesEnemy(cMap * _map,cEnemy * _enemy);

	/* 
	* Loads the image containing the player's texture 
	*/	
	void loadTexture();
	
	// Setters
	void setDirection(int direction);
	void setPlayerStatus(const int player_status);
	
	
	// Getters
	int getNumLives() const;
	cTexture* getTexturePlayer() const;
	int getFrameSeq() const;
	int getDirection() const;	
	int getPlayerStatus() const;
	float getDespx() const;
	float getDespy() const;
	float getTileSizeX() const;
	float getTileSizeY() const;
	int getPosx() const;
	int getPosy() const;
};
#endif; // __PLAYER_H__