#include "cScene.h"
#include <string.h>
#include <math.h>
#include "cGameCore.h"

cScene::cScene(void)
{
}

cScene::~cScene(void)
{
}

void cScene::Initialize()
{	
	pMap = auto_ptr<cMap>(new cMap());

	pImgHeart = auto_ptr <cTexture> (new cTexture());
	
	pImgHeart->Load("img/items.png",GL_RGBA);	

}

bool cScene::LoadLevel(const int _level)
{
	pMap->loadTexture();
	cGameCore::getInstance()->getPlayer()->loadTexture();	

	// TODO: Load enemies and items from file. Just for testing now
	for (int i=0;i<10;i++)	
	{
		items.push_back(new cItem(1,ITEM_TYPE_LIVE,100*i,400));
	}

	for (int i=0;i<5;i++)	
	{
		enemies.push_back(new cEnemy(1,ENEMY_TYPE_SKELETON,400*i,250));
	}
	
	return true;
}

void inline cScene::drawString (const char * _s)
{
	 unsigned int i;
	 for (i=0; i<strlen(_s); i++)
	 {
		 glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, _s[i]);
	 }
}

void cScene::DrawMenu(const int _pos)
{
	cTexture img;
	img.Load("img/background2.png",GL_RGBA);

	// draw the map....
	glClear(GL_COLOR_BUFFER_BIT);
	glBindTexture(GL_TEXTURE_2D,img.GetID());

	// draw the player....
	glBindTexture(GL_TEXTURE_2D,img.GetID());
	glBegin(GL_QUADS);
		glTexCoord2f(0.01,0.58);	glVertex2i(105,0);
		glTexCoord2f(0.41,0.58);	glVertex2i(535,0);
		glTexCoord2f(0.41,0.01);	glVertex2i(535,480);
		glTexCoord2f(0.01,0.01);	glVertex2i(105,480);
	glEnd();
	
	/* draw a circle from a bunch of short lines */
	float vectorY1=220-(25* _pos);
	float vectorX1=380;
	float vectorX,vectorY;
	float originX=380;
	float originY=220-(25* _pos);
	float angle;
	float radius = 10.;

	glColor3f(0.0,1.0,0.0);
	glBegin(GL_TRIANGLES);	
		for(int i=0;i<=360;i++)
		{
			angle=(float)(((double)i)/57.29577957795135);	
			vectorX=originX+(radius*(float)sin((double)angle));
			vectorY=originY+(radius*(float)cos((double)angle));		
			glVertex2d(originX,originY);
			glVertex2d(vectorX1,vectorY1);
			glVertex2d(vectorX,vectorY);
			vectorY1=vectorY;
			vectorX1=vectorX;	
		}
	glEnd();
	glColor3f(1.0,1.0,1.0);
}

void cScene::Draw() const
{
	float frame;
	float factor;	
	int contx=0;
	int conty=0;
	cTile *aux_tile;
	
	// Define the origen of the player tile, and the displacement in the image
	if(cGameCore::getInstance()->getPlayer()->getDirection()==DIR_LEFT)			
	{
		if(cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_WALK || cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_JUMPING||cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_DOWN)
		{
			factor=0.125;	
			frame = (float)cGameCore::getInstance()->getPlayer()->getFrameSeq() * 0.0625f;
		}
		else
		{
			factor=0.4;	
			frame = (float)cGameCore::getInstance()->getPlayer()->getFrameSeq() * (0.13);
		}
	}
	else if(cGameCore::getInstance()->getPlayer()->getDirection()==DIR_RIGHT)		
	{
		if(cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_WALK || cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_JUMPING||cGameCore::getInstance()->getPlayer()->getPlayerStatus()==PLAYER_STATUS_DOWN)
		{
			factor=0.;
			frame = (float)cGameCore::getInstance()->getPlayer()->getFrameSeq() * 0.0625f;
		}
		else
		{
			factor=0.25f;
			frame = (float)cGameCore::getInstance()->getPlayer()->getFrameSeq() * (0.132);
		}
	}
	
	// ***************************
	// draw the map
	// ***************************
	glClear(GL_COLOR_BUFFER_BIT);
	glBindTexture(GL_TEXTURE_2D,pMap->getTexture()->GetID());
	
	while (contx<165)
	{
		while(conty<7)
		{
			if(!pMap->isBack(contx,conty))	
			{
				aux_tile=pMap->getTile(pMap->getMapPos(contx,conty));				
				glBegin(GL_QUADS);
					glTexCoord2f(aux_tile->getTileXmin(),aux_tile->getTileYmax());	
					glVertex2i(contx*TILE_SIZE+(pMap->getCoordX())          ,conty*TILE_SIZE);

					glTexCoord2f(aux_tile->getTileXmax(),aux_tile->getTileYmax());	
					glVertex2i(contx*TILE_SIZE+TILE_SIZE+(pMap->getCoordX()),conty*TILE_SIZE);

					glTexCoord2f(aux_tile->getTileXmax(),aux_tile->getTileYmin());	
					glVertex2i(contx*TILE_SIZE+TILE_SIZE+(pMap->getCoordX()),conty*TILE_SIZE+TILE_SIZE);

					glTexCoord2f(aux_tile->getTileXmin(),aux_tile->getTileYmin());	
					glVertex2i(contx*TILE_SIZE  +(pMap->getCoordX())        ,conty*TILE_SIZE+TILE_SIZE);			
					
				glEnd();
			}
			conty++;
		}
		contx++;
		conty=0;
	}
	// ***************************
	// draw the enemies
	// ***************************
	for (int i=0;i<enemies.size();i++)
	{		
		if(enemies[i]->getEnable() == ENEMY_ENABLE)
		{
			glBindTexture(GL_TEXTURE_2D,enemies[i]->getTextureEnemy()->GetID());
			glBegin(GL_QUADS);
				glTexCoord2f(enemies[i]->getInix() , enemies[i]->getEndy());		
				glVertex2i(enemies[i]->getPosx()   , enemies[i]->getPosy());

				glTexCoord2f(enemies[i]->getEndx()                          ,enemies[i]->getEndy());		
				glVertex2i(enemies[i]->getPosx()+ enemies[i]->getEnemyTileSize() ,enemies[i]->getPosy());

				glTexCoord2f(enemies[i]->getEndx()                         , enemies[i]->getIniy());			
				glVertex2i(enemies[i]->getPosx()+enemies[i]->getEnemyTileSize(), enemies[i]->getPosy()+enemies[i]->getEnemyTileSize());

				glTexCoord2f(enemies[i]->getInix()	,enemies[i]->getIniy());			
				glVertex2i(enemies[i]->getPosx()   ,enemies[i]->getPosy()+enemies[i]->getEnemyTileSize());

			glEnd();
		}
	}
	// ***************************
	// Draw the player
	// ***************************
	glBindTexture(GL_TEXTURE_2D,cGameCore::getInstance()->getPlayer()->getTexturePlayer()->GetID());	
	glBegin(GL_QUADS);
		glTexCoord2f(frame			,factor+cGameCore::getInstance()->getPlayer()->getDespy());		
		glVertex2i(cGameCore::getInstance()->getPlayer()->getPosx(),cGameCore::getInstance()->getPlayer()->getPosy() );

		glTexCoord2f(frame+cGameCore::getInstance()->getPlayer()->getDespx(),factor+cGameCore::getInstance()->getPlayer()->getDespy());		
		glVertex2i(cGameCore::getInstance()->getPlayer()->getPosx()+cGameCore::getInstance()->getPlayer()->getTileSizeX(),cGameCore::getInstance()->getPlayer()->getPosy());

		glTexCoord2f(frame+cGameCore::getInstance()->getPlayer()->getDespx(),factor);			
		glVertex2i(cGameCore::getInstance()->getPlayer()->getPosx()+cGameCore::getInstance()->getPlayer()->getTileSizeX(),cGameCore::getInstance()->getPlayer()->getPosy()+cGameCore::getInstance()->getPlayer()->getTileSizeY());

		glTexCoord2f(frame				,factor);			
		glVertex2i(cGameCore::getInstance()->getPlayer()->getPosx()    ,cGameCore::getInstance()->getPlayer()->getPosy()+cGameCore::getInstance()->getPlayer()->getTileSizeY());
	glEnd();


	// ***************************
	// draw the items
	// ***************************
	for (int i=0;i<items.size();i++)
	{		
		if(items[i]->getEnable() == ITEM_ENABLE)
		{
			glBindTexture(GL_TEXTURE_2D,items[i]->getTextureItem()->GetID());
			glBegin(GL_QUADS);
				glTexCoord2f(items[i]->getInix() , items[i]->getEndy());		
				glVertex2i(items[i]->getPosx()   , items[i]->getPosy());

				glTexCoord2f(items[i]->getEndx()                          ,items[i]->getEndy());		
				glVertex2i(items[i]->getPosx()+ items[i]->getItemTileSize() ,items[i]->getPosy());

				glTexCoord2f(items[i]->getEndy()                         , items[i]->getIniy());			
				glVertex2i(items[i]->getPosx()+items[i]->getItemTileSize(), items[i]->getPosy()+items[i]->getItemTileSize());

				glTexCoord2f(items[i]->getInix()	,items[i]->getIniy());			
				glVertex2i(items[i]->getPosx()   ,items[i]->getPosy()+items[i]->getItemTileSize());

			glEnd();
		}
	}
	// ***************************
	// draw the hearts to show the player's lifes
	// ***************************
	for(int i=0;i<cGameCore::getInstance()->getPlayer()->getNumLives();i++)
	{
		//glBindTexture(GL_TEXTURE_2D,items[i]->getTexture_item().GetID());
		glBindTexture(GL_TEXTURE_2D,pImgHeart->GetID());
			glBegin(GL_QUADS);
				glTexCoord2f(0.5f  , 0.4f);						
				glVertex2i(600 - (i*32+5), 428);

				glTexCoord2f(0.97f   ,0.4f);						
				glVertex2i(632- (i*32+5),428);

				glTexCoord2f(0.97f   , 0.f);							
				glVertex2i(632- (i*32+5),460 );

				glTexCoord2f(0.5f	,0.f);							
				glVertex2i(600 - (i*32+5),460);
			glEnd();
	}
}

cMap * cScene::getMap() const
{
	return pMap.get();
}
