//
//
//		Name:		cScene.h
//		Date:		01-02-2009
//		Author:		Alejandro Santiago
//		Email:		alejandro@aeonphyxius.com
//		Desc:		Scene container main class. This class encapsulates all logic
//					to draw the scene, menus,etc
//
//

#ifndef __SCENE_H__
#define __SCENE_H__

#include "cTexture.h"
#include "cPlayer.h"
#include "cMap.h"
#include "cItem.h"
#include "cEnemy.h"
#include <vector>

#define SCENE_WIDTH		32
#define SCENE_HEIGHT	32
#define SECENE_DELAY 10000

using namespace std;

class cScene
{

private:

	auto_ptr <cMap> pMap;				// Scene's Tile map
	auto_ptr <cTexture> pImgHeart;		// Lives heart texture
	vector<cItem*> items;				// Items on this scene
	vector <cEnemy*> enemies;			// enemies on this scene
	
public:
	cScene();
	~cScene();
			
	/**
	* Loads the given level into our scene
	*
	* @param _level level to be loaded
	* @return true if the load process have finalized ok
    */
	bool LoadLevel(const int _level);
	
	/**
	* Draw the current scene objects in game mode
	*
    */
	void Draw() const;
	
	/**
	* Initializes the current scene object before start using it !!!!!
	*	
    */
	void Initialize();

	/**
	* Draw the current scene objects in menu mode
	*
	* @param _pos menu position
    */
	void DrawMenu(const int _pos);

	/**
	* This methods draws a string on the screen. As could be called very often, and
	* its content is better accesible for performance issues (not really the case in this program)
	*
	* @param _s string to be drawn in screen
    */
	void inline drawString (const char * _s);

	/**
	* Gets the current scene's map 
	*
	* @return Map object of this scene
    */
	cMap * getMap() const;

	// Getters
	vector <cItem*> getItems() {return items;}
	vector <cEnemy*>getEnemies(){return enemies;}
};
#endif; // __SCENE_H__